from enum import IntEnum


class Network(IntEnum):
    MAINNET = 1
    ROPSTEN = 3
    RINKEBY = 4
    GOERLI = 5
    POLYGON = 137
    BSC = 56
    AVALANCHE = 43114
    ARBITRUM = 42161
    OPTIMISM = 10
    FANTOM = 250



from .utils import hex_to_int


class Receipt:
    def __init__(self, dict):
        statusVal = dict.get('status')
        self.status = statusVal == "0x1" if type(
            statusVal) == str else statusVal
        self.root = dict.get('root')
        self.gas_used = hex_to_int(dict.get('gasUsed', dict.get('gas_used')))
        self.cumulative_gas_used = hex_to_int(dict.get(
            'cumulativeGasUsed', dict.get('cumulative_gas_used')))
        self.logs_bloom = dict.get('logsBloom', dict.get('logs_bloom'))
        self.logs = list(map(lambda t: Log(t), dict.get('logs', [])))
        self.contract_address = dict.get(
            'contractAddress', dict.get('contract_address'))
        self.block_number = dict.get('blockNumber', dict.get('block_number'))
        self.block_hash = dict.get('blockHash', dict.get('block_hash'))
        self.transaction_index = dict.get(
            'transactionIndex', dict.get('transaction_index'))
        self.transaction_hash = dict.get(
            'transactionHash', dict.get('transaction_hash'))


class Log:
    def __init__(self, dict):
        self.address = dict.get('address')
        self.topics = dict.get('topics', [])
        self.data = dict.get('data')
        self.log_index = dict.get('logIndex', dict.get('log_index'))
        self.block_number = dict.get('blockNumber', dict.get('block_number'))
        self.block_hash = dict.get('blockHash', dict.get('block_hash'))
        self.transaction_index = dict.get(
            'transactionIndex', dict.get('transaction_index'))
        self.transaction_hash = dict.get(
            'transactionHash', dict.get('transaction_hash'))
        self.removed = dict.get('removed')

    @property
    def logIndex(self):
        return self.log_index

    @property
    def blockNumber(self):
        return self.block_number

    @property
    def blockHash(self):
        return self.block_hash

    @property
    def transactionIndex(self):
        return self.transaction_index

    @property
    def transactionHash(self):
        return self.transaction_hash

    def __getitem__(self, key):
        return getattr(self, key)



from .finding import Finding, FindingSeverity, FindingType
from .block_event import BlockEvent
from .transaction_event import TransactionEvent, TxEventBlock
from .block import Block
from .transaction import Transaction
from .receipt import Receipt, Log
from .trace import Trace, TraceAction, TraceResult
from .event_type import EventType
from .network import Network
from .utils import get_json_rpc_url, create_block_event, create_transaction_event, get_web3_provider, keccak256, get_transaction_receipt
from web3 import Web3

web3Provider = Web3(Web3.HTTPProvider(get_json_rpc_url()))



from .event_type import EventType
from .network import Network
from .block import Block


class BlockEvent:
    def __init__(self, dict):
        typeVal = dict.get('type', "BLOCK")
        self.type = EventType[typeVal] if type(
            typeVal) == str else EventType(typeVal)
        networkVal = dict.get('network', "MAINNET")
        networkVal = int(networkVal) if type(
            networkVal) == str and networkVal.isdigit() else networkVal
        self.network = Network[networkVal] if type(
            networkVal) == str else Network(networkVal)
        self.block = Block(dict.get('block', {}))

    @property
    def block_hash(self):
        return self.block.hash

    @property
    def block_number(self):
        return self.block.number
